<?php

namespace Util;

use Exception\CustomException;
use Teknisa\Libs\Util\Utilities as LibUtilities;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\ORM\DateTime;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class Utilities {
    const GENERAL_ENTITY_NAMESPACE = '\\Teknisa\\Model\\Geral\\Entities\\';
    const TEKNISA_ENTITY_DIR = '\\Teknisa\\Model\\Teknisa\\Entities\\';
    const DATE_TIME_ZONE_SP = 'America/Sao_Paulo';

    const INVALID_TOKEN = 'INVALID_TOKEN';
    const EXPIRED_TOKEN = 'EXPIRED_TOKEN';

    public static function getCurrentDate() {
        $currentDate = new DateTime();
        $currentDate->setTimezone(new \DateTimeZone(self::DATE_TIME_ZONE_SP));
        return $currentDate;
    }

    public static function getValueInFilterDataSourceByName(array &$filters,$nameFilter,$remove = false){
        foreach ($filters as $key => $filter) {
            $name = $filter['columnName'];
            $values = $filter['value'];
            if($name == $nameFilter ){
                if($remove){
                    unset($filters[$key]);
                }
                return $values;
            }
        }
        return null;
    }

    public static function getRandomPassword(){
        $length = 7;
        $characters = 'abcdefghijkmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890@#$';
        $charactersLength = strlen($characters);
        $randomPassword = '';
        for($i = 1; $i <= $length; $i++){
            $rand = mt_rand(1, $charactersLength);
            $randomPassword .= $characters[$rand-1];
        }
        return $randomPassword;
    }

    /**
     * @param \DateTime $initialDate
     * @param \DateTime $endDate
     * @return int
     */
    public static function getDateIntervalInSeconds(\DateTime $initialDate, \DateTime $endDate) {
        $interval = $initialDate->diff($endDate);

        $intervalDays     = (int)$interval->format('%R%a');
        $intervalHours    = (int)$interval->format('%R%h');
        $intervalMinutes  = (int)$interval->format('%R%i');
        $intervalSeconds  = (int)$interval->format('%R%s');

        $intervalHours    = $intervalHours   + $intervalDays  * 24;
        $intervalMinutes  = $intervalMinutes + $intervalHours * 60;
        return $intervalSeconds + $intervalMinutes * 60;
    }

    public static function validateIpInRange($ip, $lowIp) {
        $lowIpArr = explode('.', $lowIp);
        $lowIpArr = array_reverse($lowIpArr);
        foreach($lowIpArr as &$lowIpPos) {
            if($lowIpPos == '0') {
                $lowIpPos = '255';
            } else {
                break;
            }
        }
        $highIp = join('.', array_reverse($lowIpArr));

        $lowIpLong  = ip2long($lowIp);
        $highIpLong = ip2long($highIp);
        $ipLong     = ip2long($ip);

        return $ipLong <= $highIpLong && $ipLong >= $lowIpLong;
    }

    public static function isIpv4($ip) {
        return strpos($ip, ":") === false ? true : false;
    }

    public static function getUseDomainFilterParameter() {
        return InstanceManager::getInstance()->getParameter('use_domain_filter');
    }

    public static function getDomainFilterParameter() {
        return InstanceManager::getInstance()->getParameter('domain_filter');
    }

    public static function getTmpPathParameter() {
        return InstanceManager::getInstance()->getParameter('tmp_path');
    }

    public static function getServerIdParameter() {
        return InstanceManager::getInstance()->getParameter('server_id');
    }

    public static function getOAccCustomParameter() {
        return InstanceManager::getInstance()->getParameter('oAccCustom');
    }

    /** @return |boolean */
    public static function getEnableSaveFSAPIkeyInDBParameter()
    {
        try {
            return InstanceManager::getInstance()->getParameter('enableSaveFSAPIkeyInDB');
        } catch(\Exception $e) {
            return false;    
        }
    }

    public static function getAccessProfileFunctionParameter() {
        $function = null;
        try {
            $function = InstanceManager::getInstance()->getParameter('accessProfileFunction');
        } catch(\Exception $e) {}
        return $function;
    }

    public static function getAccessFilePath($organizationId) {
        $path = LibUtilities::getProductBasePath() . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'oAccTek';
        if (!is_dir($path)) {
            mkdir($path, 0666);
        }
        return $path . DIRECTORY_SEPARATOR . 'oAcc' . $organizationId . '.tek';
    }

    public static function getMacAddress() {
        if(strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
            exec('ipconfig /all', $ipconfig_output, $ipConfigResult);
            if ($ipConfigResult == 0) {
                $physicalAddresses = array();
                foreach ($ipconfig_output as &$line) {
                    $matches = array();
                    if (preg_match("/([[:blank:]][[:xdigit:]]{2}-[[:xdigit:]]{2}-[[:xdigit:]]{2}-[[:xdigit:]]{2}-[[:xdigit:]]{2}-[[:xdigit:]]{2})$/i", $line, $matches)) {
                        $physicalAddresses[] = trim($matches[0]);
                    }
                }
                return $physicalAddresses[0];
            }
            return $ipConfigResult;
        } else {
            exec("cat /sys/class/net/$(ip route show default | grep -v docker | grep -v veth | awk '/default/ {print $5}')/address", $result);
            if (empty($result)) {
                return "notfound";
            }
            return trim($result[0]);
        }
    }

    /**
     * @param $token
     * @param $requestType
     * @param $data
     * @param $dataSetResultPos
     * @param $privateKey
     * @param $url
     * @param bool $keepConnected
     * @param bool $isThrowNotConnectedErrors
     * @return array
     * @throws \Exception
     */
    public static function callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $url, $keepConnected = false, $isThrowNotConnectedErrors = false) {
        set_time_limit(180);
        $notConnectCUrlErrorCodes = array(
            7,  //Failed to connect to host or proxy
            28, //Operation timeout
            35, //SSL connection error
            89, //No connection available
        );
        $header = array();
        $header[] = 'OAuth-Token: ' . $token;
        $header[] = 'OAuth-SecretId: ' . $privateKey;
        if($keepConnected) {
            $header[] = 'OAuth-KeepConnected: ' . $keepConnected;
        }
        $header[] = 'Content-Type: application/json';
        $header[] = 'Content-Length: ' . strlen(json_encode($data));

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_URL, $url . $requestType);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);

        $resultCurl = curl_exec($ch);
        $result     = json_decode($resultCurl, true);
        $error      = curl_error($ch);
        $errorCode  = curl_errno($ch);
        curl_close($ch);

        if (!empty($result['error'])) {
            $errorCode = isset($result['errorCode']) ? $result['errorCode'] : $errorCode;
            throw new \Exception($result['error'], $errorCode);
        } else if (!empty($error)) {
            if(in_array($errorCode, $notConnectCUrlErrorCodes) && !$isThrowNotConnectedErrors) {
                return null;
            }
            throw new \Exception($error, $errorCode);
        } else if(is_null($result) && $isThrowNotConnectedErrors) {
            throw new \Exception($resultCurl, $errorCode);
        } else {
            return isset($result['dataset'][$dataSetResultPos]) ? $result['dataset'][$dataSetResultPos] : null;
        }
    }

    public static function getDbHostInfo(){
        return RepositoryInstance::getOAccTekMgr()->getDbHostInfo();
    }

    public static function getLibToken($url) {
        $integration = LibUtilities::getLoginIntegration();
        $param = array(
            'CLIENT_ID' => $integration['clientId'],
            'CLIENT_SECRET' => $integration['clientSecret']
        );
        $token = self::callWebService(null, '/lib_requestToken', $param, 'token', $integration['clientSecret'], $url);
        return $token[0];
    }

    public static function createQueryFiltered($filters, $tableAlias = null) {
        $filterSQL = '';
        foreach ($filters as $filter) {
            $name = $filter['columnName'];
            $operator = $filter['operator'];
            $values = $filter['value'];
            if (($name != '*' && !empty($values)) || $values == 0) {
                if ($operator == 'BETWEEN') {
                    if ($values[0] && $values[1]) {
                        $values = "TO_DATE( '$values[0] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')
                                AND TO_DATE( '$values[1] 23:59:59' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else if ($values[0]) {
                        $operator = '>';
                        $values = " TO_DATE( '$values[0] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else if ($values[1]) {
                        $operator = '<';
                        $values = " TO_DATE( '$values[1] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else {
                        $values = "";
                    }

                    if ($values || ($values && ($values[0] || $values[1]))) {
                        if (!empty($tableAlias)) {
                            $filterSQL .= "AND $tableAlias.$name $operator $values";
                        } else {
                            $filterSQL .= "AND $name $operator $values";
                        }
                    }
                } else {

                    if (is_array($values) && $operator != 'MAPPED_LIKE_ALL') {
                        $stringValues = "('" . implode($values, "','") . "')";
                        $values = $stringValues;
                    }
                    if ($operator == 'LIKE') {
                        if (!empty($tableAlias)) {
                            $values = " UPPER('" . $values . "')";
                            $name = "UPPER($tableAlias.$name)";
                        } else {
                            $values = "UPPER('" . $values . "')";
                            $name = "UPPER($name)";
                        }
                    }
                    if ($operator == 'LIKE_ALL') {
                        if (!empty($tableAlias)) {
                            $values = "UPPER('" . $values . "')";
                        } else {
                            $values = "UPPER('" . $values . "')";
                        }

                        $columnList = explode('|', $name);
                        $likes = array();
                        foreach ($columnList as $column) {
                            $likes[] = "UPPER($column) LIKE $values";
                        }
                        $filterLikeAll = implode($likes, ' OR ');
                        $filterSQL .= "AND ( $filterLikeAll ) ";

                    } else if ($operator == 'LIKE') {
                        $filterSQL .= "AND $name $operator $values";
                    } else if ($operator == 'IN') {
                        $filterSQL .= "AND $name $operator $values";
                    } else if ($operator == 'NOT IN') {
                        $filterSQL .= "AND $name $operator ($values)";
                    } else if ($operator == 'MAPPED_LIKE_ALL') {
                        foreach($values[0] as $index => $mappedLikeAllValues) {
                            if($index == '_ALL') {
                                $values = "UPPER('" . $mappedLikeAllValues . "')";

                                $columnList = explode('|', explode('#', $name)[0]);
                                $likes = array();
                                foreach ($columnList as $column) {
                                    if (empty($tableAlias)) {
                                        $likes[] = "UPPER($column) LIKE $values";
                                    } else {
                                        $likes[] = "UPPER($tableAlias.$column) LIKE $values";
                                    }
                                }
                                $filterLikeAll = implode($likes, ' OR ');
                                $filterSQL .= " AND ( $filterLikeAll )";
                            } else {
                                $stringValues = "('" . implode($mappedLikeAllValues, "','") . "')";
                                if (!empty($tableAlias)) {
                                    $filterSQL .= " AND $tableAlias.$index IN $stringValues ";
                                } else {
                                    $filterSQL .= " AND $index IN $stringValues ";
                                }
                            }
                        }
                    } else {
                        if (!empty($tableAlias)) {
                            $filterSQL .= "AND $tableAlias.$name $operator $values";
                        } else {
                            if (is_numeric($values)){
                                $filterSQL .= "AND $name $operator $values";
                            } else {
                                $filterSQL .= "AND $name $operator '$values'";
                            }
                        }
                    }
                }
            }
        }
        return $filterSQL;
    }

    /**
     * @param CustomException $e
     * @return Error
     */
    public static function getErrorObjectFromException(CustomException $e) {
        $error = array (
            'CODE'   => $e->getMessage(),
            'PARAMS' => $e->getParams()
        );
        return new Error(json_encode($error), $e->getCode(), $e->getTraceAsString());
    }

    public static function arrayRecursiveDiff($aArray1, $aArray2)
    {
        $aReturn = array();

        foreach ($aArray1 as $mKey => $mValue) {
            if (array_key_exists($mKey, $aArray2)) {
                if (is_array($mValue)) {
                    $aRecursiveDiff = self::arrayRecursiveDiff($mValue, $aArray2[$mKey]);
                    if (count($aRecursiveDiff)) { $aReturn[$mKey] = $aRecursiveDiff; }
                } else {
                    if ($mValue != $aArray2[$mKey]) {
                        $aReturn[$mKey] = $mValue;
                    }
                }
            } else {
                $aReturn[$mKey] = $mValue;
            }
        }
        return $aReturn;
    }
}
